--[[
Name: ZoneInfo 2
Revision: $Rev: 117 $
Author: Odica; based on the ZoneInfo module of Cartographer 2 by ckknight
SVN: svn://svn.wowace.com/wow/libtourist-3-0/mainline/trunk
Description: Addon that will show levels and other information about the zone being displayed on the World Map.
License: MIT
]]

local function trace(msg)
--	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo: "..msg)
end

if not ZoneInfo then
	ZoneInfo = LibStub("AceAddon-3.0"):NewAddon("ZoneInfo", "AceHook-3.0")
	local ZoneInfo, self = ZoneInfo, ZoneInfo
end

local ZoneInfoDataProviderMixin = CreateFromMixins(MapCanvasDataProviderMixin)
local Tourist = LibStub("LibTourist-3.0")

local t = {}
local frameHeight = 0

local fishingSkillsLoaded = false

local GAME_LOCALE = GetLocale()

local translations = {
	koKR = {
		["ZoneInfo"] = "지역 정보",
		["Transports"] = "교통",
		["Instances"] = "인스턴스",
		["Flight nodes"] = "비행 노드",
		["Fishing"] = "낚시",
		["Herbalism"] = "약초채집",
		["Mining"] = "채광",
		["Skinning"] = "무두질",
		["man"] = "명",
		["Travel advice"] = "여행 조언",
	},
	deDE = {
		["ZoneInfo"] = "Zoneninfo",
		["Transports"] = "Transporte",
		["Instances"] = "Instanzen",
		["Flight nodes"] = "Flugknoten",
		["Fishing"] = "Angeln",
		["Herbalism"] = "Kräuterkunde",
		["Mining"] = "Bergbau",
		["Skinning"] = "Kürschnerei",
		["man"] = "Spieler",
		["Travel advice"] = "Reisetipps",
	},
	frFR = {
		["ZoneInfo"] = "Infos de zone",
		["Transports"] = "Transports",
		["Instances"] = "Instances",
		["Flight nodes"] = "Nœuds de vol",
		["Fishing"] = "Pêche",
		["Herbalism"] = "Herboristerie",
		["Mining"] = "Minage",
		["Skinning"] = "Dépeçage",
		["man"] = "j",
		["Travel advice"] = "Conseils de voyage",
	},
	esES = {
		["ZoneInfo"] = "Información de la Zona",
		["Transports"] = "Transportes",
		["Instances"] = "Instancias",
		["Flight nodes"] = "Nodos de vuelo",
		["Fishing"] = "Pesca",
		["Herbalism"] = "Herboristería",
		["Mining"] = "Minería",
		["Skinning"] = "Desuello",
		["man"] = "hombres",
		["Travel advice"] = "Consejos de viaje",
	},
	zhTW = {
		["ZoneInfo"] = "地區資訊",
		["Transports"] = "運輸工具",
		["Instances"] = "副本",
		["Flight nodes"] = "飛行節點",
		["Fishing"] = "釣魚",
		["Herbalism"] = "草藥學",
		["Mining"] = "採礦",
		["Skinning"] = "剝皮",
		["man"] = "人",
		["Travel advice"] = "旅遊建議",
	},
	zhCN = {
		["ZoneInfo"] = "区域信息",
		["Transports"] = "运输",
		["Instances"] = "副本",
		["Flight nodes"] = "飞行节点",
		["Fishing"] = "钓鱼",
		["Herbalism"] = "草药学",
		["Mining"] = "采矿",
		["Skinning"] = "剥皮",
		["man"] = "人",
		["Travel advice"] = "旅游建议",
	},
	ruRU = {
		["ZoneInfo"] = "Инфо о Зонах",
		["Transports"] = "Транспорты",
		["Instances"] = "Подземелья",
		["Flight nodes"] = "Летные узлы",
		["Fishing"] = "Рыбная ловля",
		["Herbalism"] = "Травничество",
		["Mining"] = "Горное дело",
		["Skinning"] = "Снятие шкур",
		["man"] = "чел",
		["Travel advice"] = "Советы путешественникам",
	},
}

local function L(tag)
	if translations[GAME_LOCALE] then
		return translations[GAME_LOCALE][tag] or tag
	else
		return tag
	end
end

local function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

local function GetTextScale(panelSize)
	return panelSize * 0.8, panelSize * 0.5
end

local templateLoaded = false
local function ZoneInfoPanel_Close()
	Settings.ShowBackground = ZoneInfo_chkShowBackground:GetChecked()
	Settings.BackgroundOpacity = ZoneInfo_sldBackgroundOpacity:GetValue() / 100
	Settings.PanelSize = ZoneInfo_sldPanelSize:GetValue() / 100
	
	Settings.ShowFishing = ZoneInfo_chkShowFishing:GetChecked()
	Settings.ShowHerbalism = ZoneInfo_chkShowHerbalism:GetChecked()
	Settings.ShowMining = ZoneInfo_chkShowMining:GetChecked()
	Settings.ShowSkinning = ZoneInfo_chkShowSkinning:GetChecked()
	Settings.ShowBattlePets = ZoneInfo_chkShowBattlePets:GetChecked()
	Settings.ShowTransports = ZoneInfo_chkShowTransports:GetChecked()
	Settings.ShowRoadConnections = ZoneInfo_chkShowRoadConnections:GetChecked()
	Settings.ShowInstances = ZoneInfo_chkShowInstances:GetChecked()
	Settings.ShowFlightnodes = ZoneInfo_chkShowFlightnodes:GetChecked()
	Settings.ShowTravelAdvice = ZoneInfo_chkShowTravelAdvice:GetChecked()

	if templateLoaded == true then
		-- Apply the settings from the template that are not in the settings dialog
		Settings.LockMovablePanel = GlobalSettings.Template.LockMovablePanel
		Settings.PanelPositionX = GlobalSettings.Template.PanelPositionX
		Settings.PanelPositionY = GlobalSettings.Template.PanelPositionY
		Settings.ShowFrame = GlobalSettings.Template.ShowFrame
	end

	if Settings.AttachPanelToWorldMap ~= ZoneInfo_chkAttachPanelToWorldMap:GetChecked() then
		if ZoneInfo_chkAttachPanelToWorldMap:GetChecked() == true then
			-- Attach to map turned on
			Settings.AttachPanelToWorldMap = true
			ZoneInfoDataProviderMixin:ResetPanelPosition()  -- Position in top left corner of the world map
		else
			-- Attach turned off
			Settings.AttachPanelToWorldMap = false
			if templateLoaded == false then
				-- Unlock
				Settings.LockMovablePanel = false
				-- Get and apply current position
				local _, _, _, xOfs, yOfs = ZoneInfoDataProviderMixin.frame:GetPoint() -- Get position offsets related to the screen
				ZoneInfoDataProviderMixin.frame:ClearAllPoints()
				ZoneInfoDataProviderMixin.frame:SetPoint(
					"TOPLEFT",
					UIParent,
					xOfs,
					yOfs
				)
			end
		end
	end

	if templateLoaded == true and Settings.AttachPanelToWorldMap == false then
		-- Apply position from template
		ZoneInfoDataProviderMixin.frame:ClearAllPoints()
		ZoneInfoDataProviderMixin.frame:SetPoint(
			"TOPLEFT",
			UIParent,
			Settings.PanelPositionX,
			Settings.PanelPositionY
		)
	end

	if Settings.ShowBackground == true then
		ZoneInfoDataProviderMixin.frame.texture:Show()
	else
		ZoneInfoDataProviderMixin.frame.texture:Hide()
	end
	
	if Settings.ShowFrame == true then
		ZoneInfoDataProviderMixin:OpenPanel()
	else
		ZoneInfoDataProviderMixin:HidePanel()
	end

	if Settings.ShowFrame == false or Settings.AttachPanelToWorldMap == true then
		ZoneInfo_cmdLockMovablePanel:Hide()
	else
		ZoneInfo_cmdLockMovablePanel:Show()
	end	

	if Settings.LockMovablePanel == true then
		ZoneInfo_cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Locked-Up")
	else
		ZoneInfo_cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Unlocked-Up")
	end	
	ZoneInfoDataProviderMixin.frame:SetMovable(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)
	ZoneInfoDataProviderMixin.frame:EnableMouse(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)

	local headerScale, detailsScale = GetTextScale(Settings.PanelSize)
	ZoneInfoDataProviderMixin.headerText:SetScale(headerScale)
	ZoneInfoDataProviderMixin.detailsText:SetScale(detailsScale)
	ZoneInfo_cmdShowFrame:SetScale(Settings.PanelSize)
	ZoneInfo_cmdLockMovablePanel:SetScale(Settings.PanelSize * 1.25)

	if Settings.ShowFishing and fishingSkillsLoaded == false then
		Tourist:LoadFishingSkills() -- Doesn't seem to work here?
		fishingSkillsLoaded = true		
	end	

	-- Reset flag
	templateLoaded = false
end

local function ZoneInfoPanel_Cancel()
	-- Reset options UI
	ZoneInfo_chkShowBackground:SetChecked(Settings.ShowBackground)
	ZoneInfo_sldBackgroundOpacity:SetValue(Settings.BackgroundOpacity * 100)
	ZoneInfo_sldPanelSize:SetValue(Settings.PanelSize * 100)
	ZoneInfo_chkAttachPanelToWorldMap:SetChecked(Settings.AttachPanelToWorldMap)
	ZoneInfo_cmdResetPanelPosition:SetEnabled(not ZoneInfo_chkAttachPanelToWorldMap:GetChecked())
	ZoneInfo_chkShowFishing:SetChecked(Settings.ShowFishing)
	ZoneInfo_chkShowHerbalism:SetChecked(Settings.ShowHerbalism)
	ZoneInfo_chkShowMining:SetChecked(Settings.ShowMining)
	ZoneInfo_chkShowSkinning:SetChecked(Settings.ShowSkinning)
	ZoneInfo_chkShowBattlePets:SetChecked(Settings.ShowBattlePets)
	ZoneInfo_chkShowTransports:SetChecked(Settings.ShowTransports)
	ZoneInfo_chkShowRoadConnections:SetChecked(Settings.ShowRoadConnections)
	ZoneInfo_chkShowInstances:SetChecked(Settings.ShowInstances)
	ZoneInfo_chkShowFlightnodes:SetChecked(Settings.ShowFlightnodes)
	ZoneInfo_chkShowTravelAdvice:SetChecked(Settings.ShowTravelAdvice)
end


local function CheckSettings(settings)
	if settings == nil then settings = {} end

	if settings.ShowBackground == nil then settings.ShowBackground = true end
	if settings.BackgroundOpacity == nil or settings.BackgroundOpacity < 0 or settings.BackgroundOpacity > 1 then settings.BackgroundOpacity = 0.5 end
	if settings.PanelSize == nil or settings.PanelSize < 0.4 or settings.PanelSize > 1.2 then settings.PanelSize = 0.8 end
	if settings.AttachPanelToWorldMap == nil then settings.AttachPanelToWorldMap = true end
	if settings.ShowFishing == nil then settings.ShowFishing = true end
	if settings.ShowHerbalism == nil then settings.ShowHerbalism = true end
	if settings.ShowMining == nil then settings.ShowMining = true end
	if settings.ShowSkinning == nil then settings.ShowSkinning = true end
	if settings.ShowBattlePets == nil then settings.ShowBattlePets = true end
	if settings.ShowTransports == nil then settings.ShowTransports = true end
	if settings.ShowRoadConnections == nil then settings.ShowRoadConnections = true end
	if settings.ShowInstances == nil then settings.ShowInstances = true end
	if settings.ShowFlightnodes == nil then settings.ShowFlightnodes = true end
	if settings.ShowTravelAdvice  == nil then settings.ShowTravelAdvice = true end

	if settings.LockMovablePanel == nil then settings.LockMovablePanel = false end
	if settings.PanelPositionX == nil then settings.PanelPositionX = 10 end
	if settings.PanelPositionY == nil then settings.PanelPositionY = -35 end
	if settings.ShowFrame == nil then settings.ShowFrame = true end
	
	return settings
end



function ZoneInfoDataProviderMixin:SaveTemplate()
	-- Copy settings from UI to template
	if GlobalSettings.Template == nil then GlobalSettings.Template = {} end
	
	GlobalSettings.Template.ShowBackground = ZoneInfo_chkShowBackground:GetChecked()
	GlobalSettings.Template.BackgroundOpacity = ZoneInfo_sldBackgroundOpacity:GetValue() / 100
	GlobalSettings.Template.PanelSize = ZoneInfo_sldPanelSize:GetValue() / 100
	GlobalSettings.Template.AttachPanelToWorldMap = ZoneInfo_chkAttachPanelToWorldMap:GetChecked()
	GlobalSettings.Template.ShowFishing = ZoneInfo_chkShowFishing:GetChecked()
	GlobalSettings.Template.ShowHerbalism = ZoneInfo_chkShowHerbalism:GetChecked()
	GlobalSettings.Template.ShowMining = ZoneInfo_chkShowMining:GetChecked()
	GlobalSettings.Template.ShowSkinning = ZoneInfo_chkShowSkinning:GetChecked()
	GlobalSettings.Template.ShowBattlePets = ZoneInfo_chkShowBattlePets:GetChecked()
	GlobalSettings.Template.ShowTransports = ZoneInfo_chkShowTransports:GetChecked()
	GlobalSettings.Template.ShowRoadConnections = ZoneInfo_chkShowRoadConnections:GetChecked()
	GlobalSettings.Template.ShowInstances = ZoneInfo_chkShowInstances:GetChecked()
	GlobalSettings.Template.ShowFlightnodes = ZoneInfo_chkShowFlightnodes:GetChecked()
	GlobalSettings.Template.ShowTravelAdvice = ZoneInfo_chkShowTravelAdvice:GetChecked()
	
	GlobalSettings.Template.LockMovablePanel = Settings.LockMovablePanel
	GlobalSettings.Template.PanelPositionX = Settings.PanelPositionX
	GlobalSettings.Template.PanelPositionY = Settings.PanelPositionY
	GlobalSettings.Template.ShowFrame = Settings.ShowFrame
	
	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo: Template saved")
end

function ZoneInfoDataProviderMixin:LoadTemplate()
	-- Copy settings from template to UI
	ZoneInfo_chkShowBackground:SetChecked(GlobalSettings.Template.ShowBackground)
	ZoneInfo_sldBackgroundOpacity:SetValue(GlobalSettings.Template.BackgroundOpacity * 100)
	ZoneInfo_sldPanelSize:SetValue(GlobalSettings.Template.PanelSize * 100)
	ZoneInfo_chkAttachPanelToWorldMap:SetChecked(GlobalSettings.Template.AttachPanelToWorldMap)
	ZoneInfo_cmdResetPanelPosition:SetEnabled(not ZoneInfo_chkAttachPanelToWorldMap:GetChecked())
	ZoneInfo_chkShowFishing:SetChecked(GlobalSettings.Template.ShowFishing)
	ZoneInfo_chkShowHerbalism:SetChecked(GlobalSettings.Template.ShowHerbalism)
	ZoneInfo_chkShowMining:SetChecked(GlobalSettings.Template.ShowMining)
	ZoneInfo_chkShowSkinning:SetChecked(GlobalSettings.Template.ShowSkinning)
	ZoneInfo_chkShowBattlePets:SetChecked(GlobalSettings.Template.ShowBattlePets)
	ZoneInfo_chkShowTransports:SetChecked(GlobalSettings.Template.ShowTransports)
	ZoneInfo_chkShowRoadConnections:SetChecked(GlobalSettings.Template.ShowRoadConnections)
	ZoneInfo_chkShowInstances:SetChecked(GlobalSettings.Template.ShowInstances)
	ZoneInfo_chkShowFlightnodes:SetChecked(GlobalSettings.Template.ShowFlightnodes)
	ZoneInfo_chkShowTravelAdvice:SetChecked(GlobalSettings.Template.ShowTravelAdvice)
	
	-- Apply non visible settings when the dialog is confirmed
	templateLoaded = true
	
	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo: Template loaded")
end







function ZoneInfo:OnInitialize()
	trace("OnInitialize")
	self.name = L("ZoneInfo")
	self.title = L("ZoneInfo")
	
	if GlobalSettings == nil then GlobalSettings = {} end
	GlobalSettings.Template = CheckSettings(GlobalSettings.Template)
	Settings = CheckSettings(Settings)
	
	-- Register a panel in the Interface Addon Options GUI
	ZoneInfo.panel = CreateFrame( "Frame", "ZoneInfoPanel", UIParent );
	ZoneInfo.panel.name = L("ZoneInfo");
	ZoneInfo.panel.okay = function (self) ZoneInfoPanel_Close() end
	ZoneInfo.panel.cancel = function (self) ZoneInfoPanel_Cancel() end
	
	-- Setting: Show Background
	local chkShowBackground = CreateFrame("CheckButton", "ZoneInfo_chkShowBackground", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowBackground:SetPoint("TOPLEFT", 20, -20)
	ZoneInfo_chkShowBackgroundText:SetText(" Show background")
	chkShowBackground.tooltip = "Displays the Zone Information on a semi-transparent background"
	chkShowBackground:SetChecked(Settings.ShowBackground)
	
	-- Setting: Background Opacity
	local sldBackgroundOpacity = CreateFrame("Slider", "ZoneInfo_sldBackgroundOpacity", ZoneInfo.panel, "OptionsSliderTemplate")
	sldBackgroundOpacity:SetPoint("TOPLEFT", 20, -60)
	sldBackgroundOpacity:SetOrientation('HORIZONTAL')
	sldBackgroundOpacity:SetWidth(180)
	sldBackgroundOpacity:SetHeight(20)
	ZoneInfo_sldBackgroundOpacityLow:SetText("0%")
	ZoneInfo_sldBackgroundOpacityHigh:SetText("100%")
	ZoneInfo_sldBackgroundOpacityText:SetText("Background opacity ("..tostring(round(Settings.BackgroundOpacity * 100, 0)).."%)" )
	sldBackgroundOpacity.tooltipText = "Adjust the opacity of the background, where 0% is transparent and 100% is solid black"
	sldBackgroundOpacity:SetMinMaxValues(0, 100)
	sldBackgroundOpacity:SetValue(Settings.BackgroundOpacity * 100)
	sldBackgroundOpacity:SetScript("OnValueChanged", 
		function(self, value)
			ZoneInfo_sldBackgroundOpacityText:SetText("Background opacity ("..tostring(round(value, 0)).."%)" )
		end);
		
	-- Setting: Panel Size
	local sldPanelSize = CreateFrame("Slider", "ZoneInfo_sldPanelSize", ZoneInfo.panel, "OptionsSliderTemplate")
	sldPanelSize:SetPoint("TOPLEFT", 20, -100)
	sldPanelSize:SetOrientation('HORIZONTAL')
	sldPanelSize:SetWidth(180)
	sldPanelSize:SetHeight(20)
	ZoneInfo_sldPanelSizeLow:SetText("40%")
	ZoneInfo_sldPanelSizeHigh:SetText("120%")
	ZoneInfo_sldPanelSizeText:SetText("Panel size ("..tostring(round(Settings.PanelSize * 100, 0)).."%)" )
	sldPanelSize.tooltipText = "Adjust the size of the ZoneInfo panel"
	sldPanelSize:SetMinMaxValues(40, 120)
	sldPanelSize:SetValue(Settings.PanelSize * 100)
	sldPanelSize:SetScript("OnValueChanged", 
		function(self, value)
			ZoneInfo_sldPanelSizeText:SetText("Panel size ("..tostring(round(value, 0)).."%)" )
		end);
		
		
	-- Setting: AttachPanelToWorldMap
	local chkAttachPanelToWorldMap = CreateFrame("CheckButton", "ZoneInfo_chkAttachPanelToWorldMap", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkAttachPanelToWorldMap:SetPoint("TOPLEFT", 20, -140)
	ZoneInfo_chkAttachPanelToWorldMapText:SetText(" Attach panel to World Map")
	chkAttachPanelToWorldMap.tooltip = "Uncheck to make the panel movable"
	chkAttachPanelToWorldMap:SetChecked(Settings.AttachPanelToWorldMap)
	chkAttachPanelToWorldMap:SetScript("OnClick", 
		function(self, value)
			ZoneInfo_cmdResetPanelPosition:SetEnabled(not ZoneInfo_chkAttachPanelToWorldMap:GetChecked())
		end);
		
	-- Button: Reset Position
	local cmdResetPanelPosition = CreateFrame("Button", "ZoneInfo_cmdResetPanelPosition", ZoneInfo.panel, "OptionsButtonTemplate");
	cmdResetPanelPosition:SetPoint("TOPLEFT", 240, -140)
	ZoneInfo_cmdResetPanelPositionText:SetText("Reset position")
	cmdResetPanelPosition.tooltipText = "Reset the penel position to the top left of the screen"
	cmdResetPanelPosition:SetWidth(120)
	cmdResetPanelPosition:SetHeight(22)
	cmdResetPanelPosition:SetScript("OnClick", 
		function(self, value)
			ZoneInfoDataProviderMixin:ResetPanelPosition()
		end);		
	cmdResetPanelPosition:SetEnabled(not chkAttachPanelToWorldMap:GetChecked())
				
	-- Setting: Show BattlePets
	local chkShowBattlePets = CreateFrame("CheckButton", "ZoneInfo_chkShowBattlePets", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowBattlePets:SetPoint("TOPLEFT", 20, -180)
	ZoneInfo_chkShowBattlePetsText:SetText(" Show battle pet level")
	chkShowBattlePets.tooltip = "Displays the level of the battle pets"
	chkShowBattlePets:SetChecked(Settings.ShowBattlePets)

	-- Setting: Show Fishing
	local chkShowFishing = CreateFrame("CheckButton", "ZoneInfo_chkShowFishing", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowFishing:SetPoint("TOPLEFT", 20, -210)
	ZoneInfo_chkShowFishingText:SetText(" Show fishing skill")
	chkShowFishing.tooltip = "Displays the name and level of the fishing skill required for the selected zone"
	chkShowFishing:SetChecked(Settings.ShowFishing)

	-- Setting: Show Herbalism
	local chkShowHerbalism = CreateFrame("CheckButton", "ZoneInfo_chkShowHerbalism", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowHerbalism:SetPoint("TOPLEFT", 20, -240)
	ZoneInfo_chkShowHerbalismText:SetText(" Show herbalism skill")
	chkShowHerbalism.tooltip = "Displays the name and level of the herbalism skill required for the selected zone"
	chkShowHerbalism:SetChecked(Settings.ShowHerbalism)
	
	-- Setting: Show Mining
	local chkShowMining = CreateFrame("CheckButton", "ZoneInfo_chkShowMining", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowMining:SetPoint("TOPLEFT", 20, -270)
	ZoneInfo_chkShowMiningText:SetText(" Show mining skill")
	chkShowMining.tooltip = "Displays the name and level of the mining skill required for the selected zone"
	chkShowMining:SetChecked(Settings.ShowMining)

	-- Setting: Show Skinning
	local chkShowSkinning = CreateFrame("CheckButton", "ZoneInfo_chkShowSkinning", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowSkinning:SetPoint("TOPLEFT", 20, -300)
	ZoneInfo_chkShowSkinningText:SetText(" Show skinning skill")
	chkShowSkinning.tooltip = "Displays the name and level of the skinning skill required for the selected zone"
	chkShowSkinning:SetChecked(Settings.ShowSkinning)

	-- Setting: Show Transports
	local chkShowTransports = CreateFrame("CheckButton", "ZoneInfo_chkShowTransports", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowTransports:SetPoint("TOPLEFT", 20, -330)
	ZoneInfo_chkShowTransportsText:SetText(" Show transports")
	chkShowTransports.tooltip = "Displays a list of transports like boats and zeppelins"
	chkShowTransports:SetChecked(Settings.ShowTransports)

	-- Setting: Show Road Connections
	local chkShowRoadConnections = CreateFrame("CheckButton", "ZoneInfo_chkShowRoadConnections", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowRoadConnections:SetPoint("TOPLEFT", 20, -360)
	ZoneInfo_chkShowRoadConnectionsText:SetText(" Show road connections")
	chkShowRoadConnections.tooltip = "Displays connections with adjacent zones by road"
	chkShowRoadConnections:SetChecked(Settings.ShowRoadConnections)	
	
	-- Setting: Show Instances
	local chkShowInstances = CreateFrame("CheckButton", "ZoneInfo_chkShowInstances", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowInstances:SetPoint("TOPLEFT", 20, -390)
	ZoneInfo_chkShowInstancesText:SetText(" Show instances")
	chkShowInstances.tooltip = "Displays a list of instances with their level and size"
	chkShowInstances:SetChecked(Settings.ShowInstances)		

	-- Setting: Show Flightnodes
	local chkShowFlightnodes = CreateFrame("CheckButton", "ZoneInfo_chkShowFlightnodes", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowFlightnodes:SetPoint("TOPLEFT", 20, -420)
	ZoneInfo_chkShowFlightnodesText:SetText(" Show flight nodes")
	chkShowFlightnodes.tooltip = "Displays a list of flight nodes within the selected zone"
	chkShowFlightnodes:SetChecked(Settings.ShowFlightnodes)		
	
	-- Setting: Show Travel Advice
	local chkShowTravelAdvice = CreateFrame("CheckButton", "ZoneInfo_chkShowTravelAdvice", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowTravelAdvice:SetPoint("TOPLEFT", 20, -450)
	ZoneInfo_chkShowTravelAdviceText:SetText(" Show travel advice")
	chkShowTravelAdvice.tooltip = "Suggests a travel route from your position to the selected zone using road connections and transports"
	chkShowTravelAdvice:SetChecked(Settings.ShowTravelAdvice)	
	
	
	-- Button: Save to template
	local cmdSaveTemplate = CreateFrame("Button", "ZoneInfo_cmdSaveTemplate", ZoneInfo.panel, "OptionsButtonTemplate");
	cmdSaveTemplate:SetPoint("TOPLEFT", 20, -490)
	ZoneInfo_cmdSaveTemplateText:SetText("Save to template")
	cmdSaveTemplate.tooltipText = "Save the current settings to the template so they can be used for other characters"
	cmdSaveTemplate:SetWidth(120)
	cmdSaveTemplate:SetHeight(22)
	cmdSaveTemplate:SetScript("OnClick", 
		function(self, value)
			ZoneInfoDataProviderMixin:SaveTemplate()
		end);		

	-- Button: Load template
	local cmdLoadTemplate = CreateFrame("Button", "ZoneInfo_cmdLoadTemplate", ZoneInfo.panel, "OptionsButtonTemplate");
	cmdLoadTemplate:SetPoint("TOPLEFT", 150, -490)
	ZoneInfo_cmdLoadTemplateText:SetText("Load template")
	cmdLoadTemplate.tooltipText = "Load the settings from the global template to apply them to this character"
	cmdLoadTemplate:SetWidth(120)
	cmdLoadTemplate:SetHeight(22)
	cmdLoadTemplate:SetScript("OnClick", 
		function(self, value)
			ZoneInfoDataProviderMixin:LoadTemplate()
		end);		
	
	InterfaceOptions_AddCategory(ZoneInfo.panel);	
end

function ZoneInfo:OnEnable()
    WorldMapFrame:AddDataProvider(ZoneInfoDataProviderMixin)
end

function ZoneInfo:OnDisable()
    WorldMapFrame:RemoveDataProvider(ZoneInfoDataProviderMixin)
end

function ZoneInfoDataProviderMixin:ResetPanelPosition()
	if Settings.AttachPanelToWorldMap == true then
		ZoneInfoDataProviderMixin.frame:SetPoint(
			"TOPLEFT",
			ZoneInfoDataProviderMixin:GetMap():GetCanvasContainer(),
			10,
			-35
		)
	else
		Settings.PanelPositionX = 20
		Settings.PanelPositionY = -100
		ZoneInfoDataProviderMixin.frame:SetPoint(
			"TOPLEFT",
			UIParent,
			Settings.PanelPositionX,
			Settings.PanelPositionY
		)
		if Settings.ShowFrame == false then
			-- Open panel if minimized
			ZoneInfoDataProviderMixin:OpenPanel()
		end
		if Settings.LockMovablePanel == true then
			-- Unlock it
			Settings.LockMovablePanel = false
			ZoneInfo_cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Unlocked-Up")
		end
		
		DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo 2: Panel position has been reset.")
	end
end

function ZoneInfoDataProviderMixin:OpenPanel()
	Settings.ShowFrame = true
	ZoneInfoDataProviderMixin.frame.texture:SetVertexColor(0,0,0, Settings.BackgroundOpacity)
	ZoneInfoDataProviderMixin.headerText:Show()
	ZoneInfoDataProviderMixin.detailsText:Show()
	if Settings.AttachPanelToWorldMap == false then
		ZoneInfo_cmdLockMovablePanel:Show()
	end
	ZoneInfoDataProviderMixin.frame:EnableMouse(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)
	ZoneInfo_cmdShowFrameText:SetText("<")
end

function ZoneInfoDataProviderMixin:HidePanel()
	Settings.ShowFrame = false
	ZoneInfo_cmdLockMovablePanel:Hide()
	ZoneInfoDataProviderMixin.detailsText:Hide()
	ZoneInfoDataProviderMixin.headerText:Hide()
	ZoneInfoDataProviderMixin.frame.texture:SetVertexColor(0,0,0,0)  -- transparent
	ZoneInfoDataProviderMixin.frame:EnableMouse(false)
	ZoneInfo_cmdShowFrameText:SetText("ZI")
end

function ZoneInfoDataProviderMixin:OnAdded(mapCanvas)
    MapCanvasDataProviderMixin.OnAdded(self, mapCanvas)

	if not self.frame then
		trace("Mixin:OnAdded: creating frame")
		self.frame = CreateFrame(
			"Frame",
			"ZoneInfo Panel",
			self:GetMap():GetCanvasContainer()
		)
	end

	if Settings.AttachPanelToWorldMap == true then
		-- Unmovable: Anchor to World Map
		self.frame:SetPoint(
			"TOPLEFT",
			self:GetMap():GetCanvasContainer(),
			10,
			-35
		)			
	else
		-- Movable: Anchor to screen
		self.frame:SetPoint(
			"TOPLEFT",
			UIParent,
			Settings.PanelPositionX,
			Settings.PanelPositionY
		)	
	end

	self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
	self.frame.dataProvider = self
	
	-- Panel repositioning 
	-- See notes at https://wowwiki.fandom.com/wiki/API_Frame_StopMovingOrSizing
	self.frame:SetMovable(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)
	self.frame:EnableMouse(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)
	self.frame:SetScript("OnMouseDown",
		function(self, buttonName)
			-- Handle left button clicks
			if (buttonName == "LeftButton" and Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false) then
				-- StartMoving() will de-anchor the frame from it's parent and anchor it to the screen
				-- Get the current offsets before that happens
				local _, _, _, xOfs, yOfs = ZoneInfoDataProviderMixin.frame:GetPoint()
				ZoneInfoDataProviderMixin.initialToFrameOffsetX = xOfs
				ZoneInfoDataProviderMixin.initialToFrameOffsetY = yOfs
				ZoneInfoDataProviderMixin.frame:StartMoving()
				-- Get the new offsets for the starting position of the move, need this to calculate the delta
				_, _, _, xOfs, yOfs = ZoneInfoDataProviderMixin.frame:GetPoint()
				ZoneInfoDataProviderMixin.initialToScreenOffsetX = xOfs
				ZoneInfoDataProviderMixin.initialToScreenOffsetY = yOfs

			end
		end)
	self.frame:SetScript("OnMouseUp",
		function(self, buttonName)
			if (buttonName == "LeftButton" and Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false) then
				local _, _, _, finalToScreenOffsetX, finalToScreenOffsetY = ZoneInfoDataProviderMixin.frame:GetPoint()
				ZoneInfoDataProviderMixin.frame:StopMovingOrSizing()
				-- StopMovingOrSizing() does not restore the original anchor
				-- Remove possibly changed anchor 
				ZoneInfoDataProviderMixin.frame:ClearAllPoints()
				-- Caculate delta
				local deltaX = finalToScreenOffsetX - ZoneInfoDataProviderMixin.initialToScreenOffsetX
				local deltaY = finalToScreenOffsetY - ZoneInfoDataProviderMixin.initialToScreenOffsetY
				-- Apply to initial offsets to get new offsets
				local newX = ZoneInfoDataProviderMixin.initialToFrameOffsetX + deltaX
				local newY = ZoneInfoDataProviderMixin.initialToFrameOffsetY + deltaY
				-- Re-anchor using new offsets
				ZoneInfoDataProviderMixin.frame:SetPoint(
					"TOPLEFT",
					UIParent,
					newX,
					newY
				)					
				-- Store new position
				Settings.PanelPositionX = newX
				Settings.PanelPositionY = newY
			end
		end)

	-- Background
	local bgTexture = self.frame:CreateTexture("ZoneInfoBackground")
	bgTexture:SetTexture("background.blp")
	bgTexture:SetVertexColor(0,0,0, Settings.BackgroundOpacity)
	bgTexture:SetAllPoints(self.frame)
	self.frame.texture = bgTexture
	if Settings.ShowBackground == false then
		self.frame.texture:Hide()
	end
	
	-- Minimize button
	local cmdShowFrame = CreateFrame("Button", "ZoneInfo_cmdShowFrame", self.frame, "OptionsButtonTemplate");
	cmdShowFrame:SetPoint("TOPLEFT", 0, 0)
	cmdShowFrame:SetWidth(30)
	cmdShowFrame:SetHeight(30)
	cmdShowFrame:SetScale(Settings.PanelSize)
	if Settings.ShowFrame == true then
		ZoneInfo_cmdShowFrameText:SetText("<")
	else
		ZoneInfo_cmdShowFrameText:SetText("ZI")
	end
	cmdShowFrame:SetScript("OnClick", 
		function(self, value)
			if Settings.ShowFrame == false then
				ZoneInfoDataProviderMixin:OpenPanel()
			else
				ZoneInfoDataProviderMixin:HidePanel()
			end
		end)

	-- Lock button
	local cmdLockMovablePanel = CreateFrame("BUTTON", "ZoneInfo_cmdLockMovablePanel", self.frame, "");
	cmdLockMovablePanel:SetSize(30,30)
	cmdLockMovablePanel:SetPoint("TOPLEFT", 20, 4)
	cmdLockMovablePanel:SetScale(Settings.PanelSize * 1.25)
	cmdLockMovablePanel:RegisterForClicks("AnyUp")
	cmdLockMovablePanel:SetPushedTexture("Interface\\BUTTONS\\LockButton-Unlocked-Down")  -- Note: there appears to be no image for Locked-Down for some reason
	cmdLockMovablePanel:SetScript("OnClick", function(self)
			Settings.LockMovablePanel = not Settings.LockMovablePanel
			ZoneInfoDataProviderMixin.frame:SetMovable(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)
			ZoneInfoDataProviderMixin.frame:EnableMouse(Settings.AttachPanelToWorldMap == false and Settings.LockMovablePanel == false)
			if Settings.LockMovablePanel == true then
				  ZoneInfo_cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Locked-Up")
			else
				  ZoneInfo_cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Unlocked-Up")
			end
		end)
	if Settings.LockMovablePanel == true then
		  cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Locked-Up")
	else
		  cmdLockMovablePanel:SetNormalTexture("Interface\\BUTTONS\\LockButton-Unlocked-Up")
	end
	if Settings.ShowFrame == false or Settings.AttachPanelToWorldMap == true then
		cmdLockMovablePanel:Hide()
	end



	-- Texts
	local font, size = WorldMapTextFont:GetFont()
	local headerScale, detailsScale = GetTextScale(Settings.PanelSize)

	self.headerText = self.frame:CreateFontString(nil, "OVERLAY", "WorldMapTextFont")
	self.headerText:SetFont(font, size, "OUTLINE")
	self.headerText:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 14, -40)
	self.headerText:SetWidth(1000)
	self.headerText:SetScale(headerScale)
	self.headerText:SetJustifyH("LEFT")

	self.detailsText = self.frame:CreateFontString(nil, "OVERLAY", "WorldMapTextFont")
	self.detailsText:SetFont(font, size, "OUTLINE")
	self.detailsText:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 20, -180)
	self.detailsText:SetWidth(1000)
	self.detailsText:SetScale(detailsScale)
	self.detailsText:SetJustifyH("LEFT")
	
	self.headerText:SetText("ZoneInfo")
	self.detailsText:SetText("")
	
	if Settings.ShowFrame == true then
		self.frame:EnableMouse(true)
		self.frame.texture:SetVertexColor(0,0,0, Settings.BackgroundOpacity)
		self.headerText:Show()
		self.detailsText:Show()
	else
		self.detailsText:Hide()
		self.headerText:Hide()
		self.frame.texture:SetVertexColor(0,0,0,0)  -- transparent
		self.frame:EnableMouse(false)
	end
	
	self.frame:Show()
	
	if Settings.ShowFishing then
		Tourist:LoadFishingSkills()
		fishingSkillsLoaded = true
	end
	
	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo 2 loaded.")
end

function ZoneInfoDataProviderMixin:RemoveAllData()
    self.frame:Hide()
end

function ZoneInfoDataProviderMixin:RefreshAllData(fromOnShow)
	if not self.frame then
		return
	end
	
	local playerUiMapID = C_Map.GetBestMapForUnit("player");
	local currentUiMapID = WorldMapFrame:GetMapID()
	local zoneText = Tourist:GetMapNameByIDAlt(currentUiMapID)
	local playerLevel = UnitLevel("player")

	trace("RefreshAllData: currentUiMapID: "..tostring(currentUiMapID).." = "..tostring(zoneText))

	if currentUiMapID and (Tourist:IsZoneOrInstance(currentUiMapID) or Tourist:DoesZoneHaveInstances(currentUiMapID)) and not Tourist:IsContinent(currentUiMapID) then
		-- HEADER --------------------------------------------------------------------------------
		self.headerText:SetTextColor(Tourist:GetFactionColor(currentUiMapID))
		
		local low, high = Tourist:GetLevel(currentUiMapID)	
		local levelText = ""
		if low > 0 and high > 0 then
			levelText = Tourist:GetLevelString(currentUiMapID)
			local r, g, b = Tourist:GetLevelColor(currentUiMapID)
			levelText = string.format("|cff%02x%02x%02x[%s] |cff%02x%02x%02x(%s)|r", r * 255, g * 255, b * 255, levelText, 255, 255, 255, playerLevel)
			
			local groupSizeString = Tourist:GetInstanceGroupSizeString(currentUiMapID, true)
			if groupSizeString ~= "" then
				levelText = levelText.." "..groupSizeString.." "..L("man")
			end
		else
			levelText = string.format("|cff%02x%02x%02x(%s)|r", 255, 255, 255, playerLevel)
		end
			
		if levelText == "" then
			self.headerText:SetText(zoneText)
		else
			table.insert(t,zoneText)
			table.insert(t,levelText)
			self.headerText:SetText(table.concat(t, "\n"))
			for k in pairs(t) do
				t[k] = nil
			end
		end

		-- DETAILS --------------------------------------------------------------------------------
		self.detailsText:SetText("")

		-- Determine transport connections
		local transports = {}
		local hasTransports = false
		local roadConnections = {}
		local hasRoadConnections = false
		if Settings.ShowTransports or Settings.ShowRoadConnections or Settings.ShowTravelAdvice then
			for path in Tourist:IterateBorderZones(currentUiMapID, false) do
				if not Tourist:IsZoneOrInstance(path) then
					transports[path] = true
					hasTransports = true
				else 
					if Tourist:IsZone(path) then
						roadConnections[path] = true
						hasRoadConnections = true
					end
				end
			end
		end
		local hasFlightnodes = Tourist:DoesZoneHaveFlightnodes(currentUiMapID) and Settings.ShowFlightnodes
		local hasInstances = Tourist:DoesZoneHaveInstances(currentUiMapID) and Settings.ShowInstances
		local hasTravelAdvice = (playerUiMapID ~= currentUiMapID) and (hasRoadConnections or hasTransports) and Settings.ShowTravelAdvice

		local noTable = false
			
		-- 1) Add Battle Pet level
		local hasBattlePets = false

		if Settings.ShowBattlePets then
			local bpLow, bpHigh = Tourist:GetBattlePetLevel(currentUiMapID)
			hasBattlePets = (bpLow and bpHigh)
			if hasBattlePets then
				local bpLevelString = Tourist:GetBattlePetLevelString(currentUiMapID)
				--local bp_level_text = string.format("|cff%02x%02x%02x%s: |cff%02x%02x%02x[%s]|r", 255, 255, 255, L("Battle Pets"), 255, 255, 0, bpLevelString)
				local bp_level_text = string.format("|cff%02x%02x%02x%s: |cff%02x%02x%02x[%s]|r", 255, 255, 255, AUCTION_CATEGORY_BATTLE_PETS, 255, 255, 0, bpLevelString)
				
				
				table.insert(t,bp_level_text)
				
				if Settings.ShowFishing or Settings.ShowHerbalism or Settings.ShowMining or Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes then
					table.insert(t,"")
				end
			end
		end
		
		-- 2) Add fishing level
		if Settings.ShowFishing then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetFishingSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Fishing")))
				local fishing_skill_text
				if currentLevel == 0 then -- not learned yet
					fishing_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					fishing_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,fishing_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Fishing")))	 -- Gray		
			end
			if Settings.ShowHerbalism or Settings.ShowMining or Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end
		end
		
		-- 3) Add herbalism level
		if Settings.ShowHerbalism then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetHerbalismSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Herbalism")))
				local herbalism_skill_text
				if currentLevel == 0 then -- not learned yet
					herbalism_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					herbalism_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,herbalism_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Herbalism")))	 -- Gray		
			end
			if Settings.ShowMining or Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end
		end		
		
		-- 4) Add mining level
		if Settings.ShowMining then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetMiningSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Mining")))
				local mining_skill_text
				if currentLevel == 0 then -- not learned yet
					mining_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					mining_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,mining_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Mining")))	 -- Gray		
			end
			if Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end
		end
		
		-- 5) Add skinning level
		if Settings.ShowSkinning then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetSkinningSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Skinning")))
				local skinning_skill_text
				if currentLevel == 0 then -- not learned yet
					skinning_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					skinning_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,skinning_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Skinning")))	 -- Gray		
			end
			if hasTransports or hasRoadConnections or hasInstances or hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end
		end		
		
		
		-- 6) Add transport connections
		if hasTransports and Settings.ShowTransports then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Transports")))
			for k, v in pairs(transports) do
				r, g, b = Tourist:GetFactionColor(k)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..k))
			end
			if hasRoadConnections or hasInstances or hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end
		end
	
		-- 7) Add road connections
		if hasRoadConnections and Settings.ShowRoadConnections then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Road connections")))
			for k, v in pairs(roadConnections) do
				r, g, b = Tourist:GetFactionColor(k)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..k))
			end
			if hasInstances or hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end
		end

		-- 8) Add instances
		if hasInstances then
			table.insert(t, string.format("|cffffffff%s:|r", L("Instances")))
			for instance in Tourist:IterateZoneInstances(currentUiMapID) do
				local complex = nil
				if not Tourist:IsComplex(currentUiMapID) then
					complex = Tourist:GetComplex(instance)
				end
				local low, high = Tourist:GetLevel(instance)
				local r1, g1, b1 = Tourist:GetFactionColor(instance)
				local r2, g2, b2 = Tourist:GetLevelColor(instance)
				local groupSize = Tourist:GetInstanceGroupSize(instance)
				local groupSizeString = Tourist:GetInstanceGroupSizeString(instance, true)
				local name = instance
				if complex then
					name = complex .. " - " .. instance
				end
				if Tourist:IsBattleground(instance) then
					name = BATTLEGROUND..": " .. name
				elseif Tourist:IsArena(instance) then
					name = ARENA..": " .. name
				elseif groupSize > 5 then
					name = RAID..": " .. name
				end
				if Tourist:IsArena(instance) then
					table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255))
				else
					if low == high then
						if groupSize > 0 then
							table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r " .. groupSizeString.." "..L("man"), r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
						else
							table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
						end
					else
						if groupSize > 0 then
							table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r " .. groupSizeString.." "..L("man"), r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
						else
							table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
						end
					end
				end
			end
			
			if hasFlightnodes or hasTravelAdvice then
				table.insert(t,"")
			end		
		end
		
		-- 9) Add flight nodes
		if hasFlightnodes then
			local r, g, b
			local firstNode = true		
			for node in Tourist:IterateZoneFlightnodes(currentUiMapID) do
				if firstNode == true then
					firstNode = false
					-- only add the header if there effectively are known nodes (not all nodes defined in LT are guaranteed to be returned by WoW when LT's lookup is filled)
					table.insert(t, string.format("|cffffffff%s:|r", L("Flight nodes")))
				end
				r, g, b = Tourist:GetFlightnodeFactionColor(node.faction)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..node.name))
			end
			
			if hasTravelAdvice then
				table.insert(t,"")
			end		
		end

		-- 10) Add Travel advice
		if hasTravelAdvice then
			local firstStep = true
			for step in Tourist:IteratePath(playerUiMapID, currentUiMapID) do
				r, g, b = Tourist:GetFactionColor(step)
				if firstStep == true then
					-- Show the header if an advice is available
					table.insert(t, string.format("|cffffffff%s:|r", L("Travel advice")))
					table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, tostring(step)))
					firstStep = false
				else
					table.insert(t, string.format("|cff%02x%02x%02x-> %s|r", r * 255, g * 255, b * 255, tostring(step)))
				end
			end
		end

		-- Show result
		self.detailsText:SetText(table.concat(t, "\n"))
		for k in pairs(t) do
			t[k] = nil
		end

	elseif not currentUiMapID then
		-- No map ID
		self.headerText:SetTextColor(1,1,1)
		self.headerText:SetText("??")
		self.detailsText:SetText("")
	else
		-- Continent, World map, Cosmic map
		self.headerText:SetTextColor(Tourist:GetFactionColor(currentUiMapID))
		self.headerText:SetText(zoneText)
		self.detailsText:SetText("")		
	end
	
	-- Set frame height and width
	local headerScale, detailsScale = GetTextScale(Settings.PanelSize)
	if self.detailsText:GetText() == nil then
		frameHeight = (self.headerText:GetHeight() + 45) * headerScale
	else
		frameHeight = (self.detailsText:GetHeight() + 225) * detailsScale 	
	end
	self.frame:SetHeight(frameHeight)
	self.frame:SetWidth(Settings.PanelSize * 500)
	if Settings.ShowFrame == true then
		self.frame.texture:SetVertexColor(0,0,0, Settings.BackgroundOpacity)
	end	
	
end


